package objectdbevaluation;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

public class TestApplication {

    public final static int EXECUTOR_THREAD_MAX = 800;
    public final static String DATABASE_FILE_NAME = "objectdb://localhost:6136/testDb.odb";
    public final static int NUMBER_OF_TASKS = 400;
    public final static int NUMBER_OF_WRITES = 2500;

    private AtomicInteger writeCounter = new AtomicInteger(0);
    private AtomicInteger id = new AtomicInteger(0);
    private ExecutorService threadPoolExecutor;
    private EntityManagerFactory factory;

    public TestApplication() {

        Map<String, String> properties = new HashMap<String, String>();
        properties.put("javax.persistence.jdbc.user", "admin");
        properties.put("javax.persistence.jdbc.password", "admin");
        factory = Persistence.createEntityManagerFactory(DATABASE_FILE_NAME, properties);

        threadPoolExecutor = Executors.newFixedThreadPool(EXECUTOR_THREAD_MAX);
    }

    public void executeApp() {

        long startTime = System.currentTimeMillis();

        try {
            for(int i=0; i<NUMBER_OF_TASKS; i++) {
                threadPoolExecutor.execute(createTask());
            }

            threadPoolExecutor.shutdown();
            threadPoolExecutor.awaitTermination(3, TimeUnit.MINUTES);
            factory.close();
        } catch (InterruptedException e) {
        } finally {

            long timeToComplete = System.currentTimeMillis() - startTime;
            float averageTransactionTime = (float) timeToComplete/writeCounter.intValue();

            System.out.println("Database activity complete in " + timeToComplete + " ms.");
            System.out.println("Number of database transactions: " + writeCounter.intValue());
            System.out.println("Average time (ms) per transaction: " + averageTransactionTime);
        }
    }

    private EntityWriteTask createTask() {

        EntityWriteDAO dao = new EntityWriteDAO();
        dao.setWriteCounter(writeCounter);
        dao.setFactory(factory);

        EntityWriteTask task = new EntityWriteTask();
        task.setWriteNumber(NUMBER_OF_WRITES);
        task.setSecondId(id);
        task.setDao(dao);

        return task;
    }

    public static void main(String[] args) {

        TestApplication application = new TestApplication();
        application.executeApp();
    }
}
